"""Output related functions"""
import os

import pandas as pd
from tabulate import tabulate


def combine_results(results_list):
    """
    Combine a list of dataframes containing results into a single dataframe
    """
    if not results_list:
        return pd.DataFrame()

    results = pd.concat(results_list, ignore_index=True)
    results = results.sort_values(
        by=["level_1", "level_2", "level_3", "level_4", "level_5"]
    ).reset_index(drop=True)

    return results


def write_results_to_file(results, output):
    """
    Write a dataframe containing results to a file in CSV or TSV format
    """
    if not output:
        return

    os.makedirs(os.path.dirname(output), exist_ok=True)

    ext = os.path.splitext(output)[1]
    if ext == ".csv":
        results.to_csv(output, index=False)
    elif ext == ".tsv" or ext == ".txt":
        results.to_csv(output, index=False, sep="\t")
    else:
        raise SystemExit("Output file must have extension 'txt', 'tsv', or 'csv'")


def print_results(results):
    """
    Print a dataframe containing results to the console in a formatted table
    """
    headers = ["Sample", "Level 1", "Level 2", "Level 3", "Level 4", "Level 5"]
    if not results.empty:
        print(tabulate(results, headers=headers, tablefmt="grid"))
    else:
        print("No lineages were called!")



"""
TbLG (Tuberculosis Lineage Genotyping)

A command-line tool to genotype Mycobacterium tuberculosis lineage from a VCF file

Author: Dmitry Bespiatykh
"""
import sys

import click

from .barcoding import process_vcf_files
from .utils import combine_results, print_results, write_results_to_file

CONTEXT_SETTINGS = dict(help_option_names=["-h", "--help"])


@click.command(context_settings=CONTEXT_SETTINGS)
@click.argument(
    "vcf_files", nargs=-1, type=click.Path(exists=True), metavar="<vcf_files>"
)
@click.option(
    "--output",
    "-o",
    type=click.Path(),
    help="Write results to file '.txt', '.tsv', or '.csv'.",
)
@click.pass_context
def main(ctx, vcf_files, output):
    """
    TbLG (Tuberculosis Lineage Genotyping)

    Process one or more VCF files and genotype lineages.

    VCF_FILES: One or more VCF files to be processed.
    """
    if not vcf_files and not ctx.params.get("help"):
        click.echo("Note: Please provide one or more VCF files to process\n")
        click.echo(ctx.get_help())
        ctx.exit()

    if output:
        if not output.endswith((".txt", ".tsv", ".csv")):
            raise SystemExit("Output file must have extension 'txt', 'tsv', or 'csv'")
        else:
            click.echo(f"Writing results to {output}")

    results_list = process_vcf_files(vcf_files)

    if not results_list:
        print("No valid VCF files were found!")

    results = combine_results(results_list)

    if output:
        write_results_to_file(results, output)
    else:
        print_results(results)


if __name__ == "__main__":
    sys.exit(main())



from setuptools import setup, find_namespace_packages

with open("README.md", "r") as f:
    long_description = f.read()

setup(
    name="tblg",
    version="0.1.0",
    description="A command-line tool to genotype Mycobacterium tuberculosis lineage from a VCF file",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/dbespiatykh/tblg",
    author="Dmitry Bespiatykh",
    author_email="d.bespiatykh@gmail.com",
    license="MIT",
    packages=find_namespace_packages(where="src"),
    package_dir={"": "src"},
    include_package_data=True,
    package_data={"tblg.data": ["*.tsv"]},
    entry_points={"console_scripts": ["tblg = tblg.__main__:main"]},
    install_requires=[
        "click==8.0.4",
        "numpy==1.23.5",
        "pandas==1.5.0",
        "tabulate==0.9.0",
        "tqdm==4.64.1",
    ],
    python_requires=">=3.10",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: MIT License",
        "Operating System :: POSIX",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Topic :: Scientific/Engineering :: Bio-Informatics",
    ],
)
